/*
 * Copyright (c) 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Vehicle/compositevhcl.h>

CompositeVehicle::CompositeVehicle
(const char* name, Vehicle** vhcls, int nvhcls) : Vehicle(name) {
  _nvhcls = nvhcls;
  _vhcls = new Vehicle*[_nvhcls];
  double cx = 0.0;
  double cy = 0.0;
  double cz = 0.0;
  for (int i=0; i<_nvhcls; i++) {
    _vhcls[i] = vhcls[i];
    cx += _vhcls[i]->xloc();
    cy += _vhcls[i]->yloc();
    cz += _vhcls[i]->zloc();
  }
  _loc[0] = cx/_nvhcls;
  _loc[1] = cy/_nvhcls;
  _loc[2] = cz/_nvhcls;
}

CompositeVehicle::~CompositeVehicle() {
  delete [] _vhcls;
}

void CompositeVehicle::update(Observable* obs) {

  /* compute new center and heading of composite vehicle */
  double cx = 0.0;
  double cy = 0.0;
  double cz = 0.0;
  for (int i=0; i<_nvhcls; i++) {
    cx += _vhcls[i]->xloc();
    cy += _vhcls[i]->yloc();
    cz += _vhcls[i]->zloc();
  }
  cx /= _nvhcls;
  cy /= _nvhcls;
  cz /= _nvhcls;
  _dir[0] = cx - _loc[0];
  _dir[1] = cy - _loc[1];
  _dir[2] = cz - _loc[2];
  _loc[0] = cx;
  _loc[1] = cy;
  _loc[2] = cz;

  notify();
}

