/*
 * Copyright (c) 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef route_h
#define route_h

#include <Vehicle/util.h>

class FMultiLineObj;
class Vehicle;

class Route {
public:
  
  Route(const char* name=nil, int npts=0, 
	float*x=nil, float*y=nil, float*z=nil);
  virtual ~Route();
  const char* name() const { return _name; }
  float* x() const;
  float* y() const;
  float* z() const;
  float* d() const;
  int npts() const { return _npts; }
  void clear();
  
  void append(float x, float y, float z=0);
  
  void start(Vehicle* vhcl, boolean reverse=false);
  void reset();
  void step(float meters);
  boolean finished();

  Vehicle* vehicle() { return _vhcl; }
  void vehicle(Vehicle* vhcl) { _vhcl = vhcl; }
  
 protected:
  char* _name;
  FMultiLineObj* _xy;
  FMultiLineObj* _zd;
  int _npts;
  float _curx;
  float _cury;
  float _curz;
  float _segdist;
  int _curdir;
  int _curpt;
  Vehicle* _vhcl;
};

#endif

