/*
 * Copyright (c) 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef vehicle_h
#define vehicle_h

#include <Vehicle/util.h>

class Route;
class Time;

class Vehicle : public Observer, public Observable {
public:
    Vehicle(const char* name, float x=0.0, float y=0.0, float z=0.0,
	    float dx=0.0, float dy=1.0, float dz=0.0);
    virtual ~Vehicle();

    const char* name() { return _name; }
    float& xloc() { return _loc[0]; }
    float& yloc() { return _loc[1]; }
    float& zloc() { return _loc[2]; }
    float& xdir() { return _dir[0]; }
    float& ydir() { return _dir[1]; }
    float& zdir() { return _dir[2]; }

    void move(float dx, float dy);
    void move(float dx, float dy, float dy);
    void move(float* deltas, int ndeltas);
    void jump(float x, float y);
    void jump(float x, float y, float y);
    void jump(float* loc, int n);
    void turn(float dx, float dy);
    void turn(float dx, float dy, float dy);
    void turn(float* deltas, int ndeltas);
    void turnto(float x, float y);
    void turnto(float x, float y, float y);
    void turnto(float* dir, int n);

    void forward2d(float dist =1.0);
    void backward2d(float dist =1.0);
    void rotate2d(float ang);

    float heading2d();

    void speed(float spd) { _speed = spd; }
    float speed() { return _speed; }

    Route* route() { return _rte; }

    // for simulation
    void simroute(Route* route, Time* start);

    // for capture 
    void startroute(Route*);
    void stoproute();
    void growroute();

    void tag(void*);
    void* tag();

    virtual void update(Observable*);
    void reset(float x=0.0, float y=0.0, float z=0.0,
	       float dx=0.0, float dy=1.0, float dz=0.0);

    virtual Vehicle* subvhcl(int vhclindex) { return nil; }
    virtual int nsubvhcls() { return 0; }

protected:
    float _loc[3];
    float _dir[3];
    char* _name;
    Route* _rte;
    void* _tag;
    float _speed;
    Time* _clock;
    Time* _start;
};

inline void Vehicle::tag(void* ptr) { _tag = ptr; }
inline void* Vehicle::tag() { return _tag; }

#endif

