/*
 * Copyright 1997-1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <VhclServ/vhclclasses.h>
#include <VhclServ/vhclcomps.h>
#include <Vehicle/route.h>
#include <Vehicle/vehicle.h>
#include <ComTerp/comterpserv.h>
#include <ComTerp/comvalue.h>
#include <Attribute/paramlist.h>
#include <Unidraw/Graphic/graphic.h>
#include <iostream.h>
#include <string.h>
#include <strstream.h>

#ifdef HAVE_ACE
#include <ace/SOCK_Connector.h>
#endif

/****************************************************************************/


int VhclServComp::_symid = -1;

VhclServComp::VhclServComp() : OverlayComp() {
    _gr = nil;
}

#if 0
ParamList* VhclServComp::_vehicle_params = nil;

VhclServComp::VhclServComp (istream& in, OverlayComp* parent) : OverlayComp(nil, parent) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* VhclServComp::GetParamList() {
    if (!_vehicle_params) 
	GrowParamList(_vehicle_params = new ParamList());
    return _vehicle_params;
}

void VhclServComp::GrowParamList(ParamList* pl) {
    pl->add_param("name", ParamStruct::required, &ParamList::read_string,
		  this, &_name);
    pl->add_param("other", ParamStruct::other,  &OverlayScript::ReadOther,
		  this, this);
}
#endif

VhclServComp::~VhclServComp() {
    _gr = nil;
}

Component* VhclServComp::Copy() {
    VhclServComp* vhc = new VhclServComp();
    return vhc;
}

ClassId VhclServComp::GetClassId () { return VEHICLE_COMP; }

boolean VhclServComp::IsA (ClassId id) {
    return VEHICLE_COMP == id || OverlayComp::IsA(id);
}

/*****************************************************************************/

int VhclServUpdater::_symid = -1;
ComTerpServ* VhclServUpdater::_comdraw = nil;
ComTerpServ* VhclServUpdater::comdraw() { return _comdraw; }
void VhclServUpdater::comdraw(ComTerpServ* c) { _comdraw = c; }


VhclServUpdater::VhclServUpdater (VhclServComp* subj) : OverlayScript(subj) {
    _host = nil;
    _socket = nil;
}

VhclServUpdater::~VhclServUpdater() {
#ifdef HAVE_ACE
    if (_socket && _socket->close () == -1)
            ACE_ERROR ((LM_ERROR, "%p\n", "close"));
    delete(_socket);
#endif
}

ClassId VhclServUpdater::GetClassId () { return VEHICLE_UPDATER; }

boolean VhclServUpdater::IsA (ClassId id) {
    return VEHICLE_UPDATER == id || OverlayScript::IsA(id);
}

boolean VhclServUpdater::connect(const char* host, unsigned int port) {

  if (comdraw()) {
    ostream* outp = new ostrstream();
    send_connect_commands(*outp);
    ComValue& retval = comdraw()->run(((ostrstream*)outp)->str());
    delete outp;
  } else {

    filebuf fbuf;
#ifdef HAVE_ACE
    if (port) {
      _socket = new ACE_SOCK_Stream;
      ACE_SOCK_Connector conn;
      ACE_INET_Addr addr (port, host);
      
      if (conn.connect (*_socket, addr) == -1) {
	ACE_ERROR ((LM_ERROR, "%p\n", "open"));
	return false;
      }
      
      fbuf.attach(_socket->get_handle());
    } else
#endif
      fbuf.attach(fileno(stdout));

    ostream out(&fbuf);
    send_connect_commands(out);
  }
  
  return true;
}

boolean VhclServUpdater::disconnect() {

  if (comdraw()) {
    ostream* outp = new ostrstream();
    send_disconnect_commands(*outp);
    ComValue& retval = comdraw()->run(((ostrstream*)outp)->str());
    delete outp;
  } else {
    
    filebuf fbuf;
#ifdef HAVE_ACE
    if (_socket) 
      fbuf.attach(_socket->get_handle());
    else 
#endif
      fbuf.attach(fileno(stdout));
    
    ostream out(&fbuf);
    send_disconnect_commands(out);
  }
  
#ifdef HAVE_ACE
  if (_socket && _socket->close () == -1)
    ACE_ERROR ((LM_ERROR, "%p\n", "close"));
  delete(_socket);
  _socket = nil;
#endif
}

void VhclServUpdater::Update() {
  if (comdraw()) {
    ostream* outp = new ostrstream();
    send_update_commands(*outp);
    ComValue& retval = comdraw()->run(((ostrstream*)outp)->str());
    delete outp;
  } else {
    
    filebuf fbuf;
#ifdef HAVE_ACE
    if (_socket) 
      fbuf.attach(_socket->get_handle());
    else 
#endif
      fbuf.attach(fileno(stdout));
    
    ostream out(&fbuf);
    send_update_commands(out);
  }
}

/****************************************************************************/


int VehicleComp::_symid = -1;

VehicleComp::VehicleComp(const char* name) : VhclServComp() {
    _vehicle = new Vehicle(name);
    _vehicle->tag(this);
    _vehicle->attach(this);
}

VehicleComp::VehicleComp(Vehicle* vhcl) : VhclServComp() {
    _vehicle = vhcl;
    _vehicle->tag(this);
    _vehicle->attach(this);
}

#if 0
ParamList* VehicleComp::_vehicle_params = nil;

VehicleComp::VehicleComp (istream& in, OverlayComp* parent) : VhclServComp(nil, parent) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* VehicleComp::GetParamList() {
    if (!_vehicle_params) 
	GrowParamList(_vehicle_params = new ParamList());
    return _vehicle_params;
}

void VehicleComp::GrowParamList(ParamList* pl) {
    pl->add_param("name", ParamStruct::required, &ParamList::read_string,
		  this, &_name);
    pl->add_param("other", ParamStruct::other,  &OverlayScript::ReadOther,
		  this, this);
}
#endif

VehicleComp::~VehicleComp() {
}

Component* VehicleComp::Copy() {
    VehicleComp* vhc = new VehicleComp(vehicle()->name());
    return vhc;
}

ClassId VehicleComp::GetClassId () { return VEHICLE_COMP; }

boolean VehicleComp::IsA (ClassId id) {
    return VEHICLE_COMP == id || VhclServComp::IsA(id);
}

void VehicleComp::playroute(Route* rte, boolean forward) {
  if (rte && rte->x() && rte->y()) {
    int npts = rte->npts();
    for (int i=0; i<npts; i++) {
      int loc = forward ? i : npts - i - 1;
      float x = rte->x()[loc];
      float y = rte->y()[loc];
      vehicle()->jump(x, y);
      Notify();
    }
  }
}

/*****************************************************************************/

int VehicleUpdater::_symid = -1;

VehicleUpdater::VehicleUpdater (VehicleComp* subj) : VhclServUpdater(subj) {
}

VehicleUpdater::~VehicleUpdater() {
}

ClassId VehicleUpdater::GetClassId () { return VEHICLE_UPDATER; }

boolean VehicleUpdater::IsA (ClassId id) {
    return VEHICLE_UPDATER == id || VhclServUpdater::IsA(id);
}

void VehicleUpdater::send_connect_commands(ostream& out)  {
    const char* name = ((VehicleComp*)GetSubject())->vehicle()->name();
    out << name << "=vhclview(";
    out << " :name \"" <<  name << "\"";
    out << " :gr \"drawtool(";
#if 0
    out << "  rect(-15,-25,15,25 :fillbg 1 :brush 65535,0 :fgcolor \\\"Blue\\\",0.312505,0.312505,0.968765 :bgcolor \\\"Green\\\",0.234379,0.968765,0.203128 :graypat 0.5)";
#else
    out << "  rect(-15,-25,15,25 :fillbg 1 :brush 65535,0 :fgcolor \\\"Blue\\\",0.312505,0.312505,0.968765 :bgcolor \\\"Green\\\",0.234379,0.968765,0.203128 :graypat 0.5),";
    out << "  polygon((-15,25),(0,30),(15,25) :fillbg 1 :brush 65535,0 :fgcolor \\\"Red\\\",0.312505,0.312505,0.968765 :bgcolor \\\"Green\\\",0.234379,0.968765,0.203128 :graypat 0.5)";
#endif
    out << ")\"";
    out << ")\n";
    out.flush();
#if 0
    cout << name << "=vhclview(";
    cout << " :name \"" <<  name << "\"";
    cout << " :gr \"drawtool(";
    cout << "  rect(-15,-25,15,25 :fillbg 1 :brush 65535,0 :fgcolor \\\"Blue\\\",0.312505,0.312505,0.968765 :bgcolor \\\"Green\\\",0.234379,0.968765,0.203128 :graypat 0.5)";
    cout << ")\"";
    cout << ")\n";
    cout.flush();
#endif
}

void VehicleUpdater::send_disconnect_commands(ostream& out) {
}

void VehicleUpdater::send_update_commands(ostream& out) {
    VehicleComp* vhclcomp = (VehicleComp*)GetSubject();
    Vehicle* vehicle = vhclcomp->vehicle();
    out << "moveview(" << vehicle->name() << " " 	<< (long)vehicle->xloc() << " " << (long) vehicle->yloc();
    out << " :heading " << vehicle->heading2d();
    out << " :xdir " << vehicle->xdir();
    out << " :ydir " << vehicle->ydir();
    out << ")\n";
    out.flush();
#if 0
    cout << "moveview(" << vehicle->name() << " " << (long) vehicle->xloc() << " " << (long) vehicle->yloc();
    cout << " :heading " << vehicle->heading2d();
    cout << " :xdir " << vehicle->xdir();
    cout << " :ydir " << vehicle->ydir();
    cout << ")\n";
    cout.flush();
#endif
}

/****************************************************************************/


int RouteComp::_symid = -1;

RouteComp::RouteComp(Route* rte) : VhclServComp() {
    route(rte);
}

#if 0
ParamList* RouteComp::_route_params = nil;

RouteComp::RouteComp (istream& in, OverlayComp* parent) : VhclServComp(nil, parent) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* RouteComp::GetParamList() {
    if (!_route_params) 
	GrowParamList(_route_params = new ParamList());
    return _route_params;
}

void RouteComp::GrowParamList(ParamList* pl) {
    pl->add_param("name", ParamStruct::required, &ParamList::read_string,
		  this, &_name);
    pl->add_param("other", ParamStruct::other,  &OverlayScript::ReadOther,
		  this, this);
}
#endif

RouteComp::~RouteComp() {
}

Component* RouteComp::Copy() {
    RouteComp* rtc = new RouteComp(route());
    return rtc;
}

ClassId RouteComp::GetClassId () { return ROUTE_COMP; }

boolean RouteComp::IsA (ClassId id) {
    return ROUTE_COMP == id || VhclServComp::IsA(id);
}

/*****************************************************************************/

int RouteUpdater::_symid = -1;

RouteUpdater::RouteUpdater (RouteComp* subj) : VhclServUpdater(subj) {
}

RouteUpdater::~RouteUpdater() {
}

ClassId RouteUpdater::GetClassId () { return ROUTE_UPDATER; }

boolean RouteUpdater::IsA (ClassId id) {
    return ROUTE_UPDATER == id || VhclServUpdater::IsA(id);
}

void RouteUpdater::send_connect_commands(ostream& out) {
    RouteComp* rtecomp = (RouteComp*)GetSubject();
    Route* route = rtecomp->route();
    float* xpts = route->x();
    float* ypts = route->y();
    const char* name = ((RouteComp*)GetSubject())->route()->name();
    out << name << "=rteview(";
    for (int i=0; i<route->npts(); i++) {
        if (i) out << ",";
        out.form( "%.6f,%.6f", xpts[i], ypts[i]);
    }
    out << " :name \"" <<  name << "\"";
    out << ")\n";
    out.flush();
#if 0
    cout << name << "=rteview(";
    for (int i=0; i<route->npts(); i++) {
        if (i) cout << ",";
        cout << xpts[i] << "," << ypts[i];
    }
    cout << " :name \"" <<  name << "\"";
    cout << ")\n";
    cout.flush();
#endif
}

void RouteUpdater::send_disconnect_commands(ostream& out) {
}

void RouteUpdater::send_update_commands(ostream& out) {
    send_connect_commands(out);
}

