/*
 * Copyright 1997-1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef vhclcomps_h
#define vhclcomps_h

#include <OverlayUnidraw/ovcomps.h>
#include <OverlayUnidraw/scriptview.h>
#include <ComTerp/_comutil.h>

#define classid(name) \
public: \
  static const char* classname() {return name;}\
  static int symbolid()\
    { if (_symid<0) _symid=symbol_add((char*)classname()); return _symid;} \
protected: \
  static int _symid;

class ComTerpServ;
class Observable;
class Route;
class Vehicle;
class ACE_SOCK_Stream;

class VhclServComp : public OverlayComp {
public:
    VhclServComp();
    ~VhclServComp();

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

protected:
//    ParamList* GetParamList();
//    void GrowParamList(ParamList*);
//    static ParamList* _vehicle_params;

    classid("VhclServComp");

friend class VhclServUpdater;
};

class VhclServUpdater : public OverlayScript {
public:
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    VhclServUpdater(VhclServComp* = nil);

    static ComTerpServ* comdraw();
    static void comdraw(ComTerpServ*);

    boolean connect(const char* host, unsigned int portnum);
    boolean disconnect();
    virtual void Update();

    virtual void send_connect_commands(ostream&) = 0;
    virtual void send_disconnect_commands(ostream&) = 0;
    virtual void send_update_commands(ostream&) = 0;

    classid("VhclServUpdater");

protected:
    char* _host;
    int _port;
    ACE_SOCK_Stream* _socket;
    static ComTerpServ* _comdraw;
};

class Route;

class VehicleComp : public VhclServComp {
public:
    VehicleComp(const char* name);
    VehicleComp(Vehicle*);
//    VehicleComp(istream& in, OverlayComp* parent=nil);
    virtual ~VehicleComp();

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    Vehicle* vehicle() { return _vehicle; }
    void playroute(Route*, boolean forward = true);

protected:
//    ParamList* GetParamList();
//    void GrowParamList(ParamList*);
//    static ParamList* _vehicle_params;

    Vehicle* _vehicle;

    classid("VehicleComp");

friend class VehicleUpdater;
};


class VehicleUpdater : public VhclServUpdater {
public:
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    VehicleUpdater(VehicleComp* = nil);

    boolean move(float dx, float dy, boolean absolute = false);

    virtual void send_connect_commands(ostream&);
    virtual void send_disconnect_commands(ostream&);
    virtual void send_update_commands(ostream&);

    classid("VehicleUpdater");

};

class RouteComp : public VhclServComp {
public:
    RouteComp(Route* = nil);
//    RouteComp(istream& in, OverlayComp* parent=nil);
    virtual ~RouteComp();

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    void route(Route* route) { _route = route; }
    Route* route() { return _route; }

protected:
//    ParamList* GetParamList();
//    void GrowParamList(ParamList*);
//    static ParamList* _vehicle_params;

    Route* _route;

    classid("RouteComp");

friend class RouteUpdater;
};

class RouteUpdater : public VhclServUpdater {
public:
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    RouteUpdater(RouteComp* = nil);

    boolean play();

    virtual void send_connect_commands(ostream&);
    virtual void send_disconnect_commands(ostream&);
    virtual void send_update_commands(ostream&);

    classid("RouteUpdater");

};

#endif
