/*
 * Copyright (c) 1997-1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <VhclServ/vhclclasses.h>
#include <VhclServ/vhclcomps.h>
#include <VhclServ/vhclcreator.h>
#include <VhclServ/vhclfunc.h>
#include <MapUnidraw/mapcomps.h>
#include <MapServ/mapfunc.h>
#include <Vehicle/route.h>
#include <Vehicle/compositevhcl.h>
#include <Vehicle/vehicle.h>
#include <ComTerp/comhandler.h>
#include <ComTerp/comterp.h>
#include <ComTerp/comvalue.h>
#include <ComTerp/ctrlfunc.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/unidraw.h>
#include <Attribute/aliterator.h>
#include <Attribute/attrlist.h>
#include <Time/Time.h>
#include <Time/obstime.h>

#include <iostream.h>

/*****************************************************************************/

implementTable(VehicleTable,int,void*)
implementTable(RouteTable,int,void*)

VehicleTable* VhclFunc::_vhcltable = nil;
RouteTable* VhclFunc::_rtetable = nil;

VhclFunc::VhclFunc(ComTerp* comterp) : ComFunc(comterp) {
  if (!_vhcltable) 
      _vhcltable = new VehicleTable(100);
  if (!_rtetable) 
      _rtetable = new RouteTable(100);
}

VehicleComp* VhclFunc::findvhcl(int symid) {
    void* ptr = nil;
    vhcltable()->find(ptr, symid);
    if (ptr) {
        ComValue* valptr = (ComValue*)ptr;
        VehicleComp* vhcl = (VehicleComp*)((ComponentView*)valptr->obj_val())->GetSubject();
	return vhcl;
    } else
       return nil;
}

RouteComp* VhclFunc::findrte(int symid) {
    void* ptr = nil;
    rtetable()->find(ptr, symid);
    if (ptr) {
        ComValue* valptr = (ComValue*)ptr;
        RouteComp* rte = (RouteComp*)((ComponentView*)valptr->obj_val())->GetSubject();
	return rte;
    } else
       return nil;
}


/*****************************************************************************/

CreateVhclFunc::CreateVhclFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void CreateVhclFunc::execute() {
    ComValue namestr(stack_arg(0, true));
    reset_stack();
    VehicleComp* vhclcomp = VhclFunc::findvhcl(namestr.symbol_val());
    if (vhclcomp) return;
    const char* name = namestr.string_ptr();
    vhclcomp = new VehicleComp(name);
    VhclSimTimeFunc::simtime()->attach(vhclcomp->vehicle());
    ComValue* result = 
        new ComValue(ComValue::ObjectType, new ComponentView(vhclcomp));
    result->obj_type_ref() = VehicleComp::symbolid();
    VhclFunc::vhcltable()->insert(namestr.symbol_val(), result);
    push_stack(*result);
    VhclViewerFunc func(comterp());
    func.argcnts(1,0);
    func.execute();
}

/*****************************************************************************/

CreateCompVhclFunc::CreateCompVhclFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void CreateCompVhclFunc::execute() {
    ComValue namestr(stack_arg(0, true));
    ComValue& vhclvect = stack_arg(1, true);
    if (!vhclvect.is_type(ComValue::ArrayType) || vhclvect.array_len()==0) {
        reset_stack();
	push_stack(ComValue::nullval());
	return;
    }

    const int nvhcls = vhclvect.array_len();
    Vehicle* vhcls[nvhcls];
    ALIterator i;
    AttributeValueList* avl = vhclvect.array_val();
    avl->First(i);
    for (int j=0; j<nvhcls && !avl->Done(i); j++) {
      vhcls[j] = VhclFunc::findvhcl(avl->GetAttrVal(i)->symbol_val())->vehicle();
      avl->Next(i);
    }
    reset_stack();
    
    VehicleComp* vhclcomp = VhclFunc::findvhcl(namestr.symbol_val());
    if (vhclcomp) return;

    const char* name = namestr.string_ptr();
    CompositeVehicle* compvhcl = new CompositeVehicle(name, vhcls, nvhcls);
    vhclcomp = new VehicleComp(compvhcl);

    VhclSimTimeFunc::simtime()->attach(vhclcomp->vehicle());
    ComValue* result = 
        new ComValue(ComValue::ObjectType, new ComponentView(vhclcomp));
    result->obj_type_ref() = VehicleComp::symbolid();
    VhclFunc::vhcltable()->insert(namestr.symbol_val(), result);
    push_stack(*result);
    VhclViewerFunc func(comterp());
    func.argcnts(1,0);
    func.execute();
}

/*****************************************************************************/

CreateRouteFunc::CreateRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
  _name_symid = symbol_add("name");
}

void CreateRouteFunc::execute() {
    ComValue route(stack_arg(0));
    ComValue namestr(stack_key(_name_symid, true));
    const char* name = namestr.string_ptr();
    int rtelen;
    if (nargs()-nargskey()==1) {
        rtelen = 0;
    } else if (!route.is_type(ComValue::ArrayType)) {
        push_stack(ComValue::nullval());
	return;
    } else {
        rtelen = route.array_len();
    }
    reset_stack();

    const int npts = rtelen/2;
    float* x = npts ? new float[npts] : nil;
    float* y = npts ? new float[npts] : nil;
    if (npts) {
      AttributeValueList* avl = route.array_val();
      ALIterator i;
      avl->First(i);
      for (int j=0; j<npts && !avl->Done(i); j++) {
	x[j] = avl->GetAttrVal(i)->float_val();
	avl->Next(i);
	y[j] = avl->GetAttrVal(i)->float_val();
	avl->Next(i);
      }
    }
    
    Route* rte = new Route(name, npts, x, y);
    RouteComp* rtecomp = new RouteComp(rte);
    ComValue* result = 
        new ComValue(ComValue::ObjectType, new ComponentView(rtecomp));
    result->obj_type_ref() = RouteComp::symbolid();
    VhclFunc::rtetable()->insert(namestr.symbol_val(), result);
    push_stack(*result);
    VhclViewerFunc func(comterp());
    func.argcnts(1,0);
    func.execute();
}

/*****************************************************************************/

ConnectVhclFunc::ConnectVhclFunc(ComTerp* comterp) : VhclFunc(comterp)
{}
 
void ConnectVhclFunc::execute() {
    ComValue& vr = stack_arg(0, true);    
    VehicleComp* vhclcomp = nil;
    RouteComp* rtecomp = nil;
    if (vr.is_type(ComValue::SymbolType)) {
        vhclcomp = VhclFunc::findvhcl(vr.symbol_val());
	if (!vhclcomp)
          rtecomp = VhclFunc::findrte(vr.symbol_val());
    } else {
        boolean route = false;
        ComponentView* compview = (ComponentView*)vr.geta(VehicleComp::symbolid());
	if (!compview) {
	    route = true;
	    compview = (ComponentView*)vr.geta(RouteComp::symbolid());
	}

        if (!compview) {
	    reset_stack();
            push_stack(ComValue::nullval());
	    return;
	}
	if (!route) 
	    vhclcomp = (VehicleComp*)compview->GetSubject();
	else
	    rtecomp = (RouteComp*)compview->GetSubject();

    }

    ComValue* hostp = nil;
    ComValue* portp = nil;
    ComValue& hr = stack_arg(1);    hostp = &hr;
    ComValue& pr = stack_arg(2);    portp = &pr;

    if (!vhclcomp && !rtecomp) {
        reset_stack();
        push_stack(ComValue::nullval());
	return;
    }

    VehicleUpdater* vhclremote = nil;
    RouteUpdater* rteremote = nil;

    if (vhclcomp) {
        vhclremote = (VehicleUpdater*) vhclcomp->Create(REMOTE_UPDATER);
	vhclcomp->Attach(vhclremote);
    } else {
        rteremote = (RouteUpdater*) rtecomp->Create(REMOTE_UPDATER);
	rtecomp->Attach(rteremote);
    }

    const char* host = hostp->type() == ComValue::UnknownType 
           ? nil : hostp->string_ptr();
    unsigned int portnum;
    if (portp->type() == ComValue::UnknownType) 
        portnum = 0;
    else if (portp->type() == ComValue::IntType) 
        portnum = portp->int_val();
    else
        portnum = atoi(portp->string_ptr());

    if (vhclremote && vhclremote->connect(host, portnum)) {
        ComValue retval(ComValue::ObjectType, vhclremote);
	retval.obj_type_ref() = VehicleUpdater::symbolid();
	reset_stack();
	push_stack(retval);
    } else if (rteremote && rteremote->connect(host, portnum)) {
        ComValue retval(ComValue::ObjectType, rteremote);
	retval.obj_type_ref() = RouteUpdater::symbolid();
	reset_stack();
	push_stack(retval);
    } else {
        delete vhclremote;
        delete rteremote;
	reset_stack();
	push_stack(ComValue::nullval());
    }

}

/*****************************************************************************/

DisconnectVhclFunc::DisconnectVhclFunc(ComTerp* comterp) : VhclFunc(comterp)
{}

void DisconnectVhclFunc::execute() {
#if 0
    ComValue& vr = stack_arg(0, true);    
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vr.symbol_val());

    ComValue* hostp = nil;
    ComValue* portp = nil;
    ComValue& hr = stack_arg(1);    hostp = &hr;
    ComValue& pr = stack_arg(2);    portp = &pr;

    if (!vhclcomp) {
        reset_stack();
        push_stack(ComValue::nullval());
	return;
    }

    VehicleUpdater* vhclremote = (VehicleUpdater*) vhclcomp->Create(REMOTE_UPDATER);
    vhclcomp->Attach(vhclremote);

    const char* host = hostp->type() == ComValue::UnknownType 
           ? nil : hostp->string_ptr();
    unsigned int portnum;
    if (portp->type() == ComValue::UnknownType) 
        portnum = 0;
    else if (portp->type() == ComValue::IntType) 
        portnum = portp->int_val();
    else
        portnum = atoi(portp->string_ptr());

    if (vhclremote && vhclremote->display(host, portnum)) {
        ComValue retval(ComValue::ObjectType, vhclremote);
	retval.obj_type_ref() = VehicleUpdater::symbolid();
        reset_stack();
	push_stack(retval);
    } else {
        delete vhclremote;
        reset_stack();
	push_stack(ComValue::nullval());
    }
#endif
}

/*****************************************************************************/

MoveVhclFunc::MoveVhclFunc(ComTerp* comterp) : VhclFunc(comterp)
{
    _abs_symid = symbol_add("abs");
}

void MoveVhclFunc::execute() {
    ComValue vhclsym(stack_arg(0, true));
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());

    ComValue xcoord(stack_arg(1));
    ComValue ycoord(stack_arg(2));
    ComValue abs(stack_key(_abs_symid));

    reset_stack();

    if (!vhclcomp) {
        push_stack(ComValue::nullval());
	return;
    }

    if (abs.is_true()) 
        vhclcomp->vehicle()->jump(xcoord.float_val(), ycoord.float_val());
    else
        vhclcomp->vehicle()->move(xcoord.float_val(), ycoord.float_val());
    return;
}

/*****************************************************************************/

VhclForwardFunc::VhclForwardFunc(ComTerp* comterp) : VhclFunc(comterp)
{}

void VhclForwardFunc::execute() {
    ComValue vhclsym(stack_arg(0, true));
    ComValue dist(stack_arg(1));
    reset_stack();

    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    if (vhclcomp) {
        vhclcomp->vehicle()->forward2d(dist.float_val());
    } else {
        push_stack(ComValue::nullval());
    }
    return;
}

/*****************************************************************************/

VhclBackwardFunc::VhclBackwardFunc(ComTerp* comterp) : VhclFunc(comterp)
{}

void VhclBackwardFunc::execute() {
    ComValue vhclsym(stack_arg(0, true));
    ComValue dist(stack_arg(1));
    reset_stack();

    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    if (vhclcomp) {
        vhclcomp->vehicle()->backward2d(dist.float_val());
    } else {
        push_stack(ComValue::nullval());
    }

    return;
}

VhclRotateFunc::VhclRotateFunc(ComTerp* comterp) : VhclFunc(comterp)
{}

void VhclRotateFunc::execute() {
    ComValue vhclsym(stack_arg(0, true));
    ComValue ang(stack_arg(1));
    reset_stack();

    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    if (vhclcomp) {
        Vehicle* v = vhclcomp->vehicle();
	float angle = ang.float_val();
	v->rotate2d(angle);
    } else {
        push_stack(ComValue::nullval());
    }
    return;
}

/*****************************************************************************/

ComValue* VhclViewerFunc::_host = nil;
ComValue* VhclViewerFunc::_port = nil;

VhclViewerFunc::VhclViewerFunc(ComTerp* comterp) : VhclFunc(comterp)
{
}

void VhclViewerFunc::execute() {
    if (nargs()>=2) {
        ComValue& host = stack_arg(0);
	ComValue& port = stack_arg(1);
	if (host.is_type(ComValue::StringType)) {
	    _host = new ComValue(host);
	    _port = new ComValue(port);
	}
	reset_stack();
    } else if (nargs()==1) {
	if (!_host || !_port) return;
        push_stack(*_host);
        push_stack(*_port);
	ConnectVhclFunc func(comterp());
	func.argcnts(3,0);
	func.execute();
    } else {
        reset_stack();
	push_stack(ComValue::nullval());
    }
}

/*****************************************************************************/

StartRouteFunc::StartRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
  _name_symid = symbol_add("name");
}

void StartRouteFunc::execute() {
  ComValue rtesym(stack_arg(0, true));
  RouteComp* rtecomp = VhclFunc::findrte(rtesym.symbol_val());
  ComValue vhclsym(stack_arg(1, true));
  VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
  reset_stack();
  
  if (!vhclcomp) {
    push_stack(ComValue::nullval());
    return;
  }
  
  if (!rtecomp) {
    ComValue routestr(rtesym.symbol_val(), ComValue::StringType);
    push_stack(routestr);
    ComValue namekey(_name_symid, 1, ComValue::KeywordType);
    push_stack(namekey);
    CreateRouteFunc func(comterp());
    func.argcnts(2,1,1);
    func.execute();
    pop_stack();
    rtecomp = VhclFunc::findrte(rtesym.symbol_val());
  } else 
    rtecomp->route()->clear();
  
  vhclcomp->vehicle()->startroute(rtecomp->route());
  return;
}


/*****************************************************************************/

StopRouteFunc::StopRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void StopRouteFunc::execute() {
    ComValue rtesym(stack_arg(0, true));
    RouteComp* rtecomp = VhclFunc::findrte(rtesym.symbol_val());
    ComValue vhclsym(stack_arg(1, true));
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    reset_stack();

    if (!vhclcomp || !rtecomp) {
        push_stack(ComValue::nullval());
	return;
    }

    vhclcomp->vehicle()->stoproute();
    rtecomp->Notify();
    return;
}

/*****************************************************************************/

PlayRouteFunc::PlayRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
  _speed_symid = symbol_add("speed");
}

void PlayRouteFunc::execute() {
    ComValue rtesym(stack_arg(0, true));
    RouteComp* rtecomp = VhclFunc::findrte(rtesym.symbol_val());
    ComValue vhclsym(stack_arg(1, true));
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    ComValue speed(stack_key(_speed_symid)); 
    reset_stack();

    if (!vhclcomp || !rtecomp) {
        push_stack(ComValue::nullval());
	return;
    }

    if (!speed.is_type(ComValue::UnknownType))
      vhclcomp->vehicle()->speed(speed.double_val());
    rtecomp->route()->start(vhclcomp->vehicle());
    char cmd[BUFSIZ];
    sprintf(cmd, "steproute(%s %f)", rtecomp->route()->name(), 
	    vhclcomp->vehicle()->speed());
    ComValue stepcmd(cmd);
    push_stack(stepcmd);
    TimeExprFunc func(comterp());
    func.argcnts(1,0);
    func.execute();
    return;
}

/*****************************************************************************/

StepRouteFunc::StepRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void StepRouteFunc::execute() {
    ComValue rtesym(stack_arg(0, true));
    RouteComp* rtecomp = VhclFunc::findrte(rtesym.symbol_val());
    ComValue meters(stack_arg(1));
    reset_stack();

    if (!rtecomp) {
        push_stack(ComValue::nullval());
	return;
    }
    Route* route = rtecomp->route();
    Vehicle* vehicle = route->vehicle();
    VehicleComp* vhclcomp = vehicle ? (VehicleComp*)vehicle->tag() : nil;

    rtecomp->route()->step(meters.float_val());
    if (rtecomp->route()->finished())
      comterp()->quit();
    return;
}

/*****************************************************************************/

// Set up vehicle simulation timer
ObservableTime* VhclSimTimeFunc::_simtime = nil;

VhclSimTimeFunc::VhclSimTimeFunc(ComTerp* comterp) : VhclFunc(comterp) {

  if (!_simtime) {
    _simtime = new ObservableTime(new Time(0));
  }
  
  _abs_symid = symbol_add("abs");
  _back_symid = symbol_add("back");
  _reset_symid = symbol_add("reset");
}

void VhclSimTimeFunc::execute() {
  ComValue simtime(stack_arg(0));
  ComValue abs_flag(stack_key(_abs_symid));
  ComValue back_flag(stack_key(_back_symid));
  ComValue reset_flag(stack_key(_reset_symid));
  reset_stack();
  
  if (reset_flag.is_true()) {
    Time newtime(0);
    _simtime->time(newtime);
  }

  if (!simtime.is_type(ComValue::UnknownType)) {
    if (abs_flag.is_true()) {
      Time newtime(simtime.ulong_val());
      _simtime->time(newtime);
    } else {
      Time newtime(*_simtime->time());
      if (back_flag.is_true())
	newtime -= simtime.ulong_val();
      else
	newtime += simtime.ulong_val();
      _simtime->time(newtime);
    }
  }

  filebuf fbuf;
  int fd = comterp()->handler() ? comterp()->handler()->get_handle() : 1;
  fbuf.attach(fd>=0 ? fd : 1);
  ostream out(&fbuf);
  out << "simtime:  " << _simtime->time()->seconds() << "\n";
  out.flush();
  return;
}

/*****************************************************************************/

VhclLocFunc::VhclLocFunc(ComTerp* comterp) : VhclFunc(comterp)
{
}

void VhclLocFunc::execute() {
    ComValue vhclsym(stack_arg(0, true));
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    reset_stack();

    if (!vhclcomp) {
        push_stack(ComValue::nullval());
	return;
    }
    
    Vehicle* vhcl = vhclcomp->vehicle();

    AttributeValueList* al = new AttributeValueList();
    al->Append(new AttributeValue(vhcl->xloc()));
    al->Append(new AttributeValue(vhcl->yloc()));
    al->Append(new AttributeValue(vhcl->zloc()));
    ComValue retval(al);
    push_stack(retval);
    return;
}

/*****************************************************************************/

SimRouteFunc::SimRouteFunc(ComTerp* comterp) : VhclFunc(comterp) {
  _speed_symid = symbol_add("speed");
}

void SimRouteFunc::execute() {
    ComValue rtesym(stack_arg(0, true));
    RouteComp* rtecomp = VhclFunc::findrte(rtesym.symbol_val());
    ComValue vhclsym(stack_arg(1, true));
    VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
    ComValue start(stack_arg(2));
    ComValue speed(stack_key(_speed_symid)); 
    reset_stack();

    if (!vhclcomp || !rtecomp) {
        push_stack(ComValue::nullval());
	return;
    }

    if (!speed.is_type(ComValue::UnknownType))
      vhclcomp->vehicle()->speed(speed.double_val());
    Time starttime(start.ulong_val());
    vhclcomp->vehicle()->simroute(rtecomp->route(), &starttime);
    return;
}

