/*
 * Copyright (c) 1997-1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <ComTerp/comterpserv.h>
#include <ComTerp/ctrlfunc.h>
#include <VhclServ/vhclfunc.h>
#include <VhclServ/vhclmapfunc.h>
#include <VhclServ/vhclhandler.h>
#include <MapServ/maphandler.h>
#include <DlgUnidraw/dlgfunc.h>
#include <MapServ/mapfunc.h>

VhclservHandler::VhclservHandler(void) : ComterpHandler() {
  AddCommands(comterp_);
}

void VhclservHandler::AddCommands(ComTerp* comterp) {
    comterp->add_command("vehicle", new CreateVhclFunc(comterp));
    comterp->add_command("compvhcl", new CreateCompVhclFunc(comterp));
    comterp->add_command("connect", new ConnectVhclFunc(comterp));
    comterp->add_command("disconnect", new DisconnectVhclFunc(comterp));
    comterp->add_command("movevhcl", new MoveVhclFunc(comterp));
    comterp->add_command("timeexpr", new TimeExprFunc(comterp));
    comterp->add_command("forward2d", new VhclForwardFunc(comterp));
    comterp->add_command("backward2d", new VhclBackwardFunc(comterp));
    comterp->add_command("rotate2d", new VhclRotateFunc(comterp));
    comterp->add_command("vhclviewer", new VhclViewerFunc(comterp));
    comterp->add_command("route", new CreateRouteFunc(comterp));
    comterp->add_command("startroute", new StartRouteFunc(comterp));
    comterp->add_command("stoproute", new StopRouteFunc(comterp));
    comterp->add_command("playroute", new PlayRouteFunc(comterp));
    comterp->add_command("steproute", new StepRouteFunc(comterp));
    comterp->add_command("simtime", new VhclSimTimeFunc(comterp));
    comterp->add_command("vhclloc", new VhclLocFunc(comterp));
    comterp->add_command("simroute", new SimRouteFunc(comterp));

    MapservHandler::AddCommands(comterp);
    comterp->add_command("load100kdlg", new Load100KDLGFCFunc(comterp));
    comterp->add_command("vhclpoint", new VhclPointFeatureFunc(comterp));
    comterp->add_command("vhclconvexhull", new VhclConvexHullFunc(comterp));
    comterp->add_command("roamroads", new VhclRoamRoadsFunc(comterp));

#if 0
    // this now happens in ComterpHandler::timeoutscriptid(int)
    if (COMTERP_REACTOR::instance ()->schedule_timer
	(this, 
	 (const void *) this, 
	 ACE_Time_Value (2), 
	 ACE_Time_Value (2)) == -1)
      /* ACE_ERROR_RETURN ((LM_ERROR, 
	  		 "can'(%P|%t) t register with reactor\n"), -1) */;
    
#endif
}

