/*
 * Copyright 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <VhclServ/vhclmapcomps.h>
#include <Vehicle/vehicle.h>
#include <Map/mapfeature.h>
#include <Map/mapprim.h>
#include <OverlayUnidraw/ovhull.h>
#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>
#include <iostream.h>

/****************************************************************************/

VhclMapFeatureComp::VhclMapFeatureComp(MapFeature* mf, OverlayComp* parent) : 
MapFeatureComp(mf, parent) {
}

void VhclMapFeatureComp::update(Observable* obs) {
  Vehicle* vhcl = (Vehicle*) obs;
  float fx1 = vhcl->xloc();
  float fy1 = vhcl->yloc();

  MapFeature* feat = feature();
  if (feat->type()==MapFeature::PointType) {
    MapPoint* point = feat->point_primitive();
    point->insert_points(1, &fx1, &fy1);
    Notify();
  } else if (feat->type()==MapFeature::AreaType) {
    if (int n = vhcl->nsubvhcls()) { 
      float* fx = new float[n];
      float* fy = new float[n];
      for (int i=0; i<n; i++) {
	Vehicle* subv = vhcl->subvhcl(i);
	fx[i] = subv ? subv->xloc() : 0.0;
	fy[i] = subv ? subv->yloc() : 0.0;
      }
      MapFace* face = feat->face_primitive();
      AttributeList* al = GetAttributeList();
      Attribute* convexhullattr = al->GetAttr("convexhull");
      if (convexhullattr && convexhullattr->Value()->is_true()) {
	float* px = nil;
	float* py = nil;
	int np = ConvexHullCmd::ConvexHull(n, fx, fy, px, py);
	delete fx;
	delete fy;
	fx = px;
	fy = py;
      } 
      face->insert_pointers(n, fx, fy, nil, true);
      Notify();
    }
  } else 
    cerr << "update requested for unsupported feature type\n";
}

