/*
 * Copyright (c) 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <VhclServ/vhclcomps.h>
#include <VhclServ/vhclmapcomps.h>
#include <VhclServ/vhclmapfunc.h>
#include <Vehicle/vehicle.h>
#include <MapServ/mapfunc.h>
#include <Map/mapfeature.h>
#include <Map/mapprim.h>
#include <OverlayUnidraw/ovhull.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>

/*****************************************************************************/

VhclPointFeatureFunc::VhclPointFeatureFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void VhclPointFeatureFunc::execute() {
  ComValue vhclsym(stack_arg(0, true));
  VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
  reset_stack();

  if (!vhclcomp) {
    push_stack(ComValue::nullval());
    return;
  }
  
  Vehicle* vhcl = vhclcomp->vehicle();
  float fx1 = vhcl->xloc();
  float fy1 = vhcl->yloc();

  MapPoint* mappoint = new MapPoint();
  mappoint->insert_points(1, &fx1, &fy1);
  MapFeature* point = new MapFeature(-1, MapFeature::PointType);
  point->point_primitive(mappoint);
  mappoint->feature(point);
  VhclMapFeatureComp* fcomp = new VhclMapFeatureComp(point);
  ComponentView* view = new ComponentView(fcomp);
  ComValue result;
  result.type(AttributeValue::ObjectType);
  result.obj_ref() = view;
  result.obj_type_ref() = POINT_FEATURE;
  vhcl->attach(fcomp);
  push_stack(result);
  return;
}

/*****************************************************************************/

VhclConvexHullFunc::VhclConvexHullFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void VhclConvexHullFunc::execute() {
  ComValue vhclsym(stack_arg(0, true));
  VehicleComp* vhclcomp = VhclFunc::findvhcl(vhclsym.symbol_val());
  reset_stack();

  Vehicle* vhcl = vhclcomp->vehicle();
  int nvhcl = vhcl->nsubvhcls();
  if (!vhclcomp || !nvhcl) {
    push_stack(ComValue::nullval());
    return;
  }
  
  float fx[nvhcl];
  float fy[nvhcl];
  for (int i=0; i<nvhcl; i++) {
    Vehicle* subv = vhcl->subvhcl(i);
    fx[i] = subv->xloc();
    fy[i] = subv->yloc();
  }

  float* px = nil;
  float* py = nil;
  int np = ConvexHullCmd::ConvexHull(nvhcl, fx, fy, px, py);

  if (!np) {
    push_stack(ComValue::nullval());
    return;
  }

  MapFace* mapface = new MapFace();
  mapface->insert_pointers(np, px, py, nil, true);
  MapFeature* area = new MapFeature(-1, MapFeature::AreaType);
  area->face_primitive(mapface);
  mapface->feature(area);
  VhclMapFeatureComp* fcomp = new VhclMapFeatureComp(area);
  AttributeList* al = fcomp->GetAttributeList();
  al->add_attr("convexhull", new AttributeValue(1, AttributeValue::BooleanType));
  ComponentView* view = new ComponentView(fcomp);

  ComValue result;
  result.type(AttributeValue::ObjectType);
  result.obj_ref() = view;
  result.obj_type_ref() = AREA_FEATURE;
  vhcl->attach(fcomp);
  push_stack(result);
}

/*****************************************************************************/

VhclRoamRoadsFunc::VhclRoamRoadsFunc(ComTerp* comterp) : VhclFunc(comterp) {
}

void VhclRoamRoadsFunc::execute() {
  reset_stack();
  cerr << "not implemented yet\n";
}

