/*
 * Copyright 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in 
 * advertising or publicity pertaining to distribution of the software without 
 * specific, written prior permission.  The copyright holders make no 
 * representations about the suitability of this software for any purpose.  
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <VhclUnidraw/vhcleditor.h>
#include <VhclServ/vhclmapfunc.h>
#include <VhclServ/vhclcomps.h>
#include <VhclServ/vhclhandler.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapviewer.h>
#include <Unidraw/catalog.h>
#include <Unidraw/iterator.h>
#include <Unidraw/unidraw.h>
#include <ComTerp/comterpserv.h>

/*****************************************************************************/

VhclEditor::VhclEditor(OverlayComp* gc, OverlayKit* ok) 
: UtmEditor(false, ok)
{
    Init(gc, "VhclEditor");

}

VhclEditor::VhclEditor(boolean initflag, OverlayKit* ok) : UtmEditor(initflag, ok) {
}

VhclEditor::VhclEditor(const char* file, OverlayKit* ok)
: UtmEditor(false, ok)
{
    if (file == nil) {
	Init();

    } else {
	Catalog* catalog = unidraw->GetCatalog();
	OverlayComp* comp;

	if (catalog->Retrieve(file, (Component*&) comp)) {

	  Init(comp);

	} else {
	    Init();
	    fprintf(stderr, "vhclviewer: couldn't open %s\n", file);
	}
    }
}

void VhclEditor::Init (OverlayComp* comp, const char* name) {
  if (!comp) comp = new MapIdrawComp;
  _overlay_kit->Init(comp, name);
}

void VhclEditor::AddCommands(ComTerp* comterp) {
  UtmEditor::AddCommands(comterp);
  VhclservHandler::AddCommands(comterp);
  VhclServUpdater::comdraw((ComTerpServ*)comterp);
}

