/*
 * Copyright (c) 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * vhclviewer main program.
 */

#include <MapServ/mapservdef.h>
#include <MapUnidraw/mapcatalog.h>

#include <VhclUnidraw/vhclmapcreator.h>
#include <VhclUnidraw/vhcleditor.h>
#include <VhclServ/vhclcomps.h>
#include <VhclServ/vhclhandler.h>

#include <OverlayUnidraw/ovunidraw.h>

#include <InterViews/world.h>

#ifdef HAVE_ACE
#include <ComUnidraw/comterp-acehandler.h>
#include <MapUnidraw/map-aceimport.h>
#include <Dispatch/ace_dispatcher.h>
#endif

#include <stream.h>

/*****************************************************************************/

static PropertyData properties[] = {
    { "*VhclEditor*name", "Vehicle Map Viewer" },
    { "*VhclEditor*iconName", "vhclviewer" },
    { "*domain",  "map" },
    { "*initialbrush",  "2" },
    { "*initialfgcolor","1" },
    { "*initialbgcolor","10" },
    { "*initialfont",   "4" },
    { "*initialpattern","1" },
    { "*initialarrow", "none" },
    { "*pagewidth", "11" },
    { "*pageheight", "8.5" },
    { "*pagecols", "0" },
    { "*pagerows", "0" },
    { "*gridxincr", "8" },
    { "*gridyincr", "8" },
    { "*font1", "-*-courier-medium-r-normal-*-8-*-*-*-*-*-*-* Courier 8" },
    { "*font2", "-*-courier-medium-r-normal-*-10-*-*-*-*-*-*-* Courier 10" },
    { "*font3", "-*-courier-bold-r-normal-*-12-*-*-*-*-*-*-* Courier-Bold 12" },
    { "*font4",
	"-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-* Helvetica 12"
    },
    { "*font5",
	"-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-* Helvetica 14"
    },
    { "*font6",
	"-*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-* Helvetica-Bold 14"
    },
    { "*font7",
	"-*-helvetica-medium-o-normal-*-14-*-*-*-*-*-*-* Helvetica-Oblique 14"
    },
    { "*font8",
	"-*-times-medium-r-normal-*-12-*-*-*-*-*-*-*  Times-Roman 12"
    },
    { "*font9", "-*-times-medium-r-normal-*-14-*-*-*-*-*-*-* Times-Roman 14" } ,
    { "*font10", "-*-times-bold-r-normal-*-14-*-*-*-*-*-*-*  Times-Bold 14" },
    { "*font11",
	"-*-times-medium-i-normal-*-14-*-*-*-*-*-*-* Times-Italic 14"
    },
    { "*brush1",	"none" },
    { "*brush2",	"ffff 0" },
    { "*brush3",	"ffff 1" },
    { "*brush4",	"ffff 2" },
    { "*brush5",	"ffff 3" },
    { "*brush6",	"fff0 0" },
    { "*brush7",	"fff0 1" },
    { "*brush8",	"fff0 2" },
    { "*brush9",	"fff0 3" },
    { "*pattern1",	"none" },
    { "*pattern2",	"0.0" },
    { "*pattern3",	"1.0" },
    { "*pattern4",	"0.75" },
    { "*pattern5",	"0.5" },
    { "*pattern6",	"0.25" },
    { "*pattern7",	"1248" },
    { "*pattern8",	"8421" },
    { "*pattern9",	"f000" },
    { "*pattern10",	"8888" },
    { "*pattern11",	"f888" },
    { "*pattern12",	"8525" },
    { "*pattern13",	"cc33" },
    { "*pattern14",	"7bed" },
    { "*fgcolor1",	"Black" },
    { "*fgcolor2",	"Brown 42240 10752 10752" },
    { "*fgcolor3",	"Red" },
    { "*fgcolor4",	"Orange" },
    { "*fgcolor5",	"Yellow" },
    { "*fgcolor6",	"Green" },
    { "*fgcolor7",	"Blue" },
    { "*fgcolor8",	"Indigo 48896 0 65280" },
    { "*fgcolor9",	"Violet 20224 12032 20224" },
    { "*fgcolor10",	"White" },
    { "*fgcolor11",	"LtGray 50000 50000 50000" },
    { "*fgcolor12",	"DkGray 33000 33000 33000" },
    { "*bgcolor1",	"Black" },
    { "*bgcolor2",	"Brown 42240 10752 10752" },
    { "*bgcolor3",	"Red" },
    { "*bgcolor4",	"Orange" },
    { "*bgcolor5",	"Yellow" },
    { "*bgcolor6",	"Green" },
    { "*bgcolor7",	"Blue" },
    { "*bgcolor8",	"Indigo 48896 0 65280" },
    { "*bgcolor9",	"Violet 20224 12032 20224" },
    { "*bgcolor10",	"White" },
    { "*bgcolor11",	"LtGray 50000 50000 50000" },
    { "*bgcolor12",	"DkGray 33000 33000 33000" },
    { "*history",	"20" },
    { "*tile",          "false" },
    { "*twidth",        "512" },
    { "*theight",       "512" },
    { "*dem",           "" },
    { "*dtedlos",       "" },
#ifdef HAVE_ACE
    { "*host",          ACE_DEFAULT_SERVER_HOST },
    { "*port",          "20010" },
    { "*import",        "20011" },
    { "*camport",       "20015" },
    { "*vhclserv",      "20017" },
    { "*mapservhost",   MAPSERV_DEFAULT_HOST },
    { "*mapservport",   MAPSERV_DEFAULT_PORT_STR },
    { "*losservhost",   "localhost" },
    { "*losservport",   "20018" },
#endif
    { nil }
};

static OptionDesc options[] = {
    { "-tile", "*tile", OptionValueImplicit, "true" },
    { "-twidth", "*twidth", OptionValueNext },
    { "-theight", "*theight", OptionValueNext },
    { "-dem", "*dem", OptionValueNext },
    { "-demlos", "*dem", OptionValueNext },
    { "-dtedlos", "*dtedlos", OptionValueNext },
    { "-pagecols", "*pagecols", OptionValueNext },
    { "-pagerows", "*pagerows", OptionValueNext },
#ifdef HAVE_ACE
    { "-host", "*host", OptionValueNext },
    { "-port", "*port", OptionValueNext },
    { "-import", "*import", OptionValueNext },
    { "-camport", "*import", OptionValueNext },
    { "-vhclserv", "*vhclserv", OptionValueNext },
    { "-mapservhost", "*mapservhost", OptionValueNext },
    { "-mapservport", "*mapservport", OptionValueNext },
    { "-losservhost", "*losservhost", OptionValueNext },
    { "-losservport", "*losservport", OptionValueNext },
#endif
    { nil }
};

/*****************************************************************************/

int main (int argc, char** argv) {
#ifdef HAVE_ACE
    Dispatcher::instance(new AceDispatcher(COMTERP_REACTOR::instance()));
#endif

    VhclMapCreator creator;
    MapCatalog* catalog = new MapCatalog("vhclviewer", &creator);
    OverlayUnidraw* unidraw = new OverlayUnidraw(
        catalog, argc, argv, options, properties
    );

#ifdef HAVE_ACE
   // Acceptor factory.
    UnidrawComterpAcceptor peer_acceptor;

    const char* portstr = catalog->GetAttribute("port");
    int portnum = atoi(portstr);
    if (peer_acceptor.open 
	(ACE_INET_Addr (portnum)) == -1)
        cerr << "vhclviewer:  unable to open comterp port " << portnum << "\n";

    else if (COMTERP_REACTOR::instance ()->register_handler 
	     (&peer_acceptor, ACE_Event_Handler::READ_MASK) == -1)
        cerr << "vhclviewer:  unable to register ComterpAcceptor with ACE reactor\n";
    else
      cerr << "accepting comterp port (" << portnum << ") connections\n";

    MapImportAcceptor import_acceptor;

    const char* importstr = catalog->GetAttribute("import");
    int importnum = atoi(importstr);
    if (import_acceptor.open 
	(ACE_INET_Addr (importnum)) == -1)
        cerr << "vhclviewer:  unable to open import port " << importnum << "\n";

    else if (COMTERP_REACTOR::instance ()->register_handler 
	     (&import_acceptor, ACE_Event_Handler::READ_MASK) == -1)
        cerr << "vhclviewer:  unable to register MapImportAcceptor with ACE reactor\n";
    else
      cerr << "accepting import port (" << importnum << ") connections\n";

    // Register COMTERP_QUIT_HANDLER to receive SIGINT commands.  When received,
    // COMTERP_QUIT_HANDLER becomes "set" and thus, the event loop below will
    // exit.
    if (COMTERP_REACTOR::instance ()->register_handler 
	     (SIGINT, COMTERP_QUIT_HANDLER::instance ()) == -1)
        cerr << "vhclviewer:  unable to register quit handler with ACE reactor\n";

   VhclservAcceptor vhclserv_acceptor;

    const char* vhclservstr = catalog->GetAttribute("vhclserv");
    int vhclservnum = atoi(vhclservstr);
    if (vhclserv_acceptor.open 
	(ACE_INET_Addr (vhclservnum)) == -1)
        cerr << "vhclviewer:  unable to open vhclserv port " << vhclservnum << "\n";

    else if (COMTERP_REACTOR::instance ()->register_handler 
	     (&vhclserv_acceptor, ACE_Event_Handler::READ_MASK) == -1)
        cerr << "vhclviewer:  unable to register VhclservAcceptor with ACE reactor\n";
    else
      cerr << "accepting vhclserv port (" << vhclservnum << ") connections\n";

#endif
    int exit_status = 0;
    if (argc > 2) {
	cerr << "Usage: vhclviewer [-tile] [-twidth n] [-theight n] [-dem file] [-demlos file] [-pagecols n] [-pagerows n] [-host name] [-port n] [-import n] [-vhclserv n] [-mapservhost name] [-mapservport name] [-losservhost name] [-losservport n] [[file]\n";
	exit_status = 1;

    } else {
	const char* initial_file = (argc == 2) ? argv[1] : nil;
	VhclEditor* ed = new VhclEditor(initial_file);
	VhclServUpdater::comdraw((ComTerpServ*)ed->GetComTerp());

	unidraw->Open(ed);
	unidraw->Run();
    }

    delete unidraw;
    return exit_status;
}
